--This is the internal name of the mod.
local XBARMOD="XCompanionBar";
local dbver="0";

--Default settings and other info about the mod
getfenv()["XBarModData"][XBARMOD]={
	----REQUIRED VALUES----
	["nbuttons"]=0,
	["dbver"]=dbver,
	["dhorizontal"]=true,
	["dhidebar"]=false,
	["dorder"]="az",
	["dscale"]=1,
	["dtooltips"]=true,
	["booktype"]=BOOKTYPE_SPELL,
	["enabled"]=true,
	["nchecks"]=1,
	["check1"]="WrapBar",
	["dcheck1"]=false,
	["mchecks"]=false,
	["nsliders"]=0,
	["msliders"]=false,
	["ftexint"]=XBARMOD.."_Texture",
	["foptioncb"]=XBARMOD.."_OptionCB",
	["fbuttonid"]=XBARMOD.."_ButtonID",
	["forientcb"]=XBARMOD.."_Orient",
};

-- Save some space for information that orient callback function needs
local orientindex,orientrow;
local SETUPCOMPLETE=false;

function XCompanionBar_Setup()
	local i,v,n,_,t,m,b,c1,c2;
	local typelist={};

	if not SETUPCOMPLETE then
		typelist.mount="MOUNT";
		typelist.pet="CRITTER";

		b=0;
		for m,t in pairs(typelist) do
			n=GetNumCompanions(t);
			b=b+n;
			c1=0;
			c2=1;
			-- If we have any companions of that type, iterate through and find the pet info we need.
			if (n ~= nil) and (n>0) then
				for i=1,n do
					c1=c1+1;
					if c1>15 then
						c1=1;
						c2=c2+1;
					end
					if c1==1 then
						tinsert(XCompanionBarSpells,"#"..m..tostring(c2));
					end
					_,v,_,_,_=GetCompanionInfo(t,i);
					tinsert(XCompanionBarSpells,"&"..m.."&"..v)

					-- Now to enable it if we have just discovered this pet.
					if (XBarData[XBarOptionSet].mods.XCompanionBar) and XBarData[XBarOptionSet].mods.XCompanionBar[v] == nil then
						XBarData[XBarOptionSet].mods.XCompanionBar[v]=true;
					end
				end
				tinsert(XCompanionBarSpells," ");
			end
		end

		n=1;
		for i,v in pairs(getfenv()["XCompanionBarSpells"]) do
			if (v~=" ") then
				n=n+1;
				if (strsub(v,1,1)=="#") then
					--Menu header
					h=strsub(v,2);
					XBarModData[XBARMOD]["mcheck"..n]=h;
					XBarModData[XBARMOD]["check"..n]="";
					XBarModData[XBARMOD]["dcheck"..n]=nil;
				else
					v=XBar_Spellname(v);
					if v ~= "+" then
						XBarModData[XBARMOD]["check"..n]=v;
						XBarModData[XBARMOD]["dcheck"..n]=true;
					end
				end
			end
		end
		XBarModData[XBARMOD]["nchecks"]=n;
		XBarModData[XBARMOD]["nbuttons"]=b;

		if (b==0) then
			-- No mounts/critters, so junk it.
			print("XCompanionBar: No"..XCOMPANIONBAR_MSG2);
			getfenv()["XBarModData"][XBARMOD]["enabled"]=false;
		else
			print("XCompanionBar: "..tostring(b)..XCOMPANIONBAR_MSG2);
		end
		SETUPCOMPLETE=true;
	end
end

function XCompanionBar_OnLoad()
	if getfenv()["XBarModData"][XBARMOD]["enabled"] then
		this:RegisterEvent("PLAYER_ENTERING_WORLD");
		this:RegisterEvent("COMPANION_LEARNED");
		this:RegisterEvent("COMPANION_UPDATE");
	end
end

function XCompanionBar_OnEvent()
	local refreshdb=false;

	-- XBar has built in functionality to update its plugins
	if (event == "PLAYER_ENTERING_WORLD") then
		XCompanionBar_Setup();
		-- Let XBar know we're here so it can set up our data
		-- Double check just in case XBar hasn't got this yet.
		XBar_BuildPlayerName();
		XBar_RegisterAddon(XBARMOD);
		XBar_Update(XBARMOD);
	elseif (event == "COMPANION_LEARNED") then
		XBar_ResetSpacers(XBARMOD); -- Eliminates the "sliding buttons" effect
		XBar_Update(XBARMOD);
		print(XCOMPANIONBAR_MSG1);
	elseif (event == "COMPANION_UPDATE") then
		XBar_ResetSpacers(XBARMOD); -- Eliminates the "sliding buttons" effect
		XBar_Update(XBARMOD);
	end
end

function XCompanionBar_ButtonID(id,spellname)
	local newid=id;

	if (not getfenv()["XBarData"][getfenv()["XBarOptionSet"]]["mods"][XBARMOD][spellname]) then
		newid=nil;
	end

	return newid;
end

function XCompanionBar_OptionCB(option,value)
	XBar_ResetSpacers(XBARMOD); -- Eliminates the "sliding buttons" effect
	XBar_Update(XBARMOD);
	XBar_UpdateCooldowns(XBARMOD);
end

function XCompanionBar_Orient(index,point,anchor,anchorpoint,xdir,ydir,magnitude)
	local pt=point;
	local an=anchor;
	local ap=anchorpoint;
	local xd=xdir;
	local yd=ydir;
	local mag=magnitude;

	-- only change if we need
	if (getfenv()["XBarData"][getfenv()["XBarOptionSet"]]["mods"][XBARMOD]["WrapBar"]) then
		local bar=getfenv()["XBARMOD".."Button"..tostring(index)];

		if (index==1) then
			-- save the first button as our reference position
			orientrow=1;
			orientindex=1;
		end

		if (mag == XBAR_SPBIG) and (orientrow > 0) then
			-- This is a new spacer
			orientrow=(orientrow+1) * -1;
		end

		if (orientrow < 1) then
			-- Last button was a spacer, need to wrap
			orientrow=orientrow*-1;
			an=XBARMOD.."Button"..tostring(orientindex);
			orientindex=index;
			if (xd ~= 0) and (yd==0) then
				-- All horizontal bars will expand downwards
				pt="TOP";
				ap="BOTTOM";
				xd=0;
				yd=-1;
				mag=XBAR_SPSMALL;
			elseif (xd==0) and (yd ~= 0) then
				-- All vertical bars will expand to the right
				pt="LEFT";
				ap="RIGHT";
				xd=1;
				yd=0;
				mag=XBAR_SPSMALL;
			else
				-- If we encounter some wierd combination, we will cancel the row increment
				orientrow=orientrow-1;
			end
		end
	end
	return pt,an,ap,xd,yd,mag;
end

function XCompanionBar_Texture(texture,spellname)
	local t = texture;
	local summoned,_;

	-- Will highlight the Mount/critter the player has out
	_,summoned,_=XBar_GetCompanionInfo("MOUNT",spellname);
	if summoned then
		t="Interface\\Icons\\Spell_Nature_WispSplode";
	else
		_,summoned,_=XBar_GetCompanionInfo("CRITTER",spellname);
		if summoned then
			t="Interface\\Icons\\Spell_Nature_WispSplode";
		end
	end

	return t;
end
